//-------------------------------------------------------
//---------------------I2C Slave-----------------------
module i2c(
	SCL,
	SDA,
	
	en,
	
	rstn,
	
	clk,
	
	output_control,
	
	device_address,
	
	//interface to register
	input_shift,
	write_strobe,
	index_pointer,
	
	read_data
	);

input SCL; //asynchronous reset input
input SDA;

input en;

input rstn;
input clk;

input [6:0] device_address;

output output_control;

output [7:0] input_shift;
output write_strobe;
output reg [7:0] index_pointer;

input [7:0] read_data;

parameter [2:0] STATE_IDLE      = 3'h0,//idle
                STATE_DEV_ADDR  = 3'h1,//the slave addr match
                STATE_READ      = 3'h3,//the op=read 
                STATE_IDX_PTRW  = 3'h4,//get the index of inner-register
                STATE_WRITE     = 3'h5,//write the data in the reg 
		STATE_RDREAD2	= 3'h7;

wire		bus_rstn;

reg             start_detect;
reg             stop_detect;

reg [3:0]       bit_counter;//(from 0 to 8)9counters-> one byte=8bits and one ack=1bit
reg [7:0]       input_shift;
reg             master_ack;
reg [7:0]       output_shift;
reg             output_control;

reg sda_dl1;
reg sda_dl2;
reg sda_rqst;
reg sda_ack;
reg [17:0] timeoutclk_cnt;

reg [2:0] curr_state,next_state;
//reg [2:0] dl_state;

//parameter [6:0] device_address = 7'h55;
reg             start_resetter;
reg             stop_resetter;
wire            start_rstn = (~start_detect) & rstn;
wire		start_scln = SCL & rstn;
wire            stop_rstn = (~stop_detect) & rstn ; //detect the STOP for one cycle
wire            stop_rst = ~stop_rstn;
wire		stop_scln = SCL & rstn;
//wire            first_bit = (bit_counter == 4'h7) && !start_detect;//the 1bits one byte data
wire            lsb_bit = (bit_counter == 4'h7) ;//the 8bits one byte data
wire            ack_bit = (bit_counter == 4'h8) ;//the 9bites ack 
wire            address_detect = (input_shift[7:1] == device_address);//the input address match the slave
wire            read_write_bit = input_shift[0];// the write or read operation 0=write and 1=read
wire            write_strobe = (curr_state == STATE_WRITE) && lsb_bit;//write state and finish one byte=8bits

wire ssrstn = start_rstn & stop_rstn ;


//---------------------------------------------
//---------------detect the start--------------
//---------------------------------------------
wire sda_rstn = (~start_resetter) & rstn ;

always @ (negedge SDA or negedge sda_rstn)
        if (!sda_rstn)
                start_detect <= 1'b0;
        else
                start_detect <= SCL;

always @ (posedge SCL or negedge rstn)
        if (!rstn)
                start_resetter <= 1'b0;
        else
                start_resetter <= start_detect;

//the START just last for one cycle of SCL

//---------------------------------------------
//---------------detect the stop---------------
//---------------------------------------------

wire sdas_rstn = (~stop_resetter) & rstn;

always @ (posedge SDA or negedge sdas_rstn)
        if (!sdas_rstn)
                stop_detect <= 1'b0;
        else
                stop_detect <= SCL;

always @ (negedge SCL or negedge rstn)
        if (!rstn)
                stop_resetter <= 1'b0;
        else
                stop_resetter <= stop_detect;

//---------------------------------------------
//---------------latch the data---------------
//---------------------------------------------
always @ (negedge SCL or negedge ssrstn)
 if (!ssrstn)
  bit_counter <= 4'h0;
 else if (!en | ack_bit | start_detect)
  bit_counter <= 4'h0;
 else
  bit_counter <= bit_counter + 4'h1;

//counter to 9(from 0 to 8), one byte=8bits and one ack 
always @ (posedge SCL or negedge rstn)
  if (!rstn)
   input_shift <= 8'h00;
  else if (!ack_bit) 
   input_shift <= {input_shift[6:0], SDA};

//---------------------------------------------
//------------slave-to-master transfer---------
//---------------------------------------------
always @ (posedge SCL or negedge ssrstn)
	 if (!ssrstn)
	 	master_ack <= 1'b0;
	 else if (ack_bit)
	 	master_ack <= ~SDA;//the ack SDA is low

//---------------------------------------------
//------------state machine--------------------
//---------------------------------------------
wire st_rstn = rstn & stop_rstn & (timeoutclk_cnt != 200000 );
always @ (negedge SCL or negedge st_rstn)
	 if (!st_rstn)
	 	curr_state <= STATE_DEV_ADDR;
	 else if (start_detect)
                curr_state <= STATE_DEV_ADDR;
	 else if (en)
	 	curr_state <= next_state; 

always @ (*)
//(ack_bit)//at the 9th cycle and change the state by ACK
	case (curr_state)
	STATE_IDLE:
		next_state = start_detect ? STATE_DEV_ADDR : STATE_IDLE;
	STATE_DEV_ADDR:
		if (ack_bit)//addr don't match
			//next_state = (address_detect & read_write_bit) ? STATE_READ : STATE_IDX_PTRW;
		        next_state = (address_detect == 0)? STATE_IDLE : ((read_write_bit == 1) ? STATE_READ : STATE_IDX_PTRW);
		else
			next_state = STATE_DEV_ADDR;
	STATE_READ:
		next_state = master_ack ? STATE_READ : STATE_IDLE;
	STATE_IDX_PTRW:
		next_state = 
				 ack_bit ? STATE_WRITE : STATE_IDX_PTRW 
				;//get the index and ready to write 
	STATE_WRITE:
		next_state = STATE_WRITE;//when the state is write the state 
	default:
		next_state = STATE_IDLE;

	endcase

//---------------------------------------------
//------------Register transfers---------------
//---------------------------------------------

//-------------------for index----------------
always @ (negedge SCL or negedge rstn)
	if (!rstn)
		index_pointer <= 8'h00;
	else if (ack_bit &&
		(curr_state == STATE_IDX_PTRW)) //at the state get the inner-register index
		index_pointer <= input_shift;
	else if ( lsb_bit && ( (curr_state == STATE_WRITE) || (curr_state== STATE_READ)) )
		index_pointer <= index_pointer + 8'h01;
	else
		index_pointer <= index_pointer;

//------------------------for read-----------------------
always @ (posedge SCL or negedge bus_rstn)
        if (!bus_rstn)
		output_shift <= 8'h00;
        else if (ack_bit)//at one byte that can be load the output_shift
                output_shift <= read_data;
        else
                output_shift <= {output_shift[6:0], 1'b0};

//---------------------------------------------
//------------Output driver--------------------
//---------------------------------------------

// if rstn, or SCL transfer more than 7 bytes without start, or output_control not active more thatn 16ms
assign bus_rstn = ssrstn & (timeoutclk_cnt != 200000 ) ;

always @ (negedge SCL or negedge bus_rstn)
	if (!bus_rstn)
		output_control <= 1'b1;
        else if (start_detect)
                output_control <= 1'b1;
	else if (lsb_bit)
	// Write Ack
		output_control <=
			!(((next_state == STATE_DEV_ADDR ) && address_detect) ||
			(next_state == STATE_IDX_PTRW) || 
			(next_state == STATE_WRITE)); 
	else if (ack_bit)
		if (next_state == STATE_READ) // Ignore master ack&& master_ack) ||
			output_control <= output_shift[7];
		else
			output_control <= 1'b1;
	else if (next_state == STATE_READ)//for read send output shift to SDA
		output_control <= output_shift[7];
	else
		output_control <= 1'b1;

// Timeout counter to avoid output bus lock

//assign statelck = (curr_state == STATE_READ && 

// Syncronize sda transition from SCL to clk domain

always @ (posedge SCL or negedge rstn)
        if (!rstn) 
		sda_dl1 <= 1'b0;
	else
		sda_dl1 <= SDA;

always @ (posedge SCL or negedge rstn)
	if (!rstn) 
		sda_rqst <= 1'b0;
	else if (sda_ack)
		sda_rqst <= 1'b0;
	else if (sda_dl1 != SDA)
		sda_rqst <= 1'b1;

always @(posedge clk or negedge rstn)
        if (!rstn)
		sda_dl2 <= 1'b0;
	else
		sda_dl2 <= sda_rqst;

always @(posedge clk or negedge rstn)
	if (!rstn)
		sda_ack <= 1'b0;
	else if (sda_dl2)
		sda_ack <= 1'b1;
	else
		sda_ack <= 1'b0;

always @(posedge clk or negedge rstn)
	if (!rstn)
		timeoutclk_cnt <= 18'h00000;
	else if ( !start_rstn | sda_dl2 )
		timeoutclk_cnt <= 18'h00000;
	else if (timeoutclk_cnt < 230000)
		timeoutclk_cnt <= timeoutclk_cnt + 18'h00001;
	
endmodule


